const path = require('path');
const fs = require('fs-extra');
const cp = require('child_process');
const readPkg = require('read-pkg');
const writePkg = require('write-pkg');
const stringifyJS = require('javascript-stringify');
const { fetchFromGit } = require('./repository');
const { TEMPLATE_SETTING, REGISTRY_URL } = require('../configs');

exports.checkAndSetNpmRegistry = () => {
  const registry = cp.execSync('npm config get @cfe:registry').toString();

  if (registry !== REGISTRY_URL) {
    cp.execSync(`npm config set @cfe:registry ${REGISTRY_URL}`);
  }
};

exports.generateFiles = async (projectName, target, type) => {
  const projectPath = path.join(process.cwd(), projectName);
  const config = TEMPLATE_SETTING[target][type];
  const { git, tag } = config;

  fs.ensureDirSync(projectName);
  await fetchFromGit(projectName, git, tag);

  const pkg = readPkg.sync(projectPath);
  const appConfig = require(`${projectPath}/ytian.config.js`);

  /* eslint-disable-next-line no-underscore-dangle */
  delete pkg._id;
  delete pkg.readme;
  pkg.name = projectName;
  appConfig.name = projectName;

  writePkg.sync(projectPath, pkg);
  fs.outputFileSync(`${projectPath}/ytian.config.js`, `module.exports = ${stringifyJS(appConfig, null, 2)}`);
};
