const git = require('simple-git');
const path = require('path');
const inquirer = require('inquirer');
const program = require('commander');
const fs = require('fs-extra');

const cloneRepo = (repoPath, gitWorking, projectName) => new Promise((resolve, reject) => {
  try {
    gitWorking.clone(repoPath, projectName, null, (err) => {
      if (err) {
        reject(err);
      } else {
        resolve(true);
      }
    });
  } catch (e) {
    reject(e);
  }
});

const requestTags = (gitRepo, defaultTag) => new Promise((resolve, reject) => {
  if (program.tags) {
    gitRepo.tags(async (err, res) => {
      if (err) {
        reject(err);
      } else {
        const tags = res.all;
        if (Array.isArray(tags) && tags.length) {
          const { tag } = await inquirer.prompt([{
            name: 'tag',
            type: 'list',
            message: '请选择需要签出的tag：',
            choices: tags.map(item => ({
              name: item,
              value: item,
            })),
          }]);
          resolve(tag);
        } else {
          // No tags found
          resolve('');
        }
      }
    });
  } else if (defaultTag) {
    resolve(defaultTag);
  } else {
    reject(new Error('No "defaultTag" configured'));
  }
});

const checkoutTag = (gitRepo, tagSelected) => new Promise((resolve, reject) => {
  if (tagSelected) {
    console.log(`正在签出tag “${tagSelected}”...`);
    try {
      gitRepo.checkout(tagSelected, (err) => {
        // We resolve it no matter what
        resolve(!err);
      });
    } catch (e) {
      reject(e);
    }
  } else {
    console.log('未指定tag，使用HEAD');
    resolve(false);
  }
});

exports.fetchFromGit = (projectName, repoPath, defaultTag) => (
  new Promise(async (resolve, reject) => {
    const localPath = process.cwd();
    const gitWorking = git(localPath);
    console.log('正在请求仓库...');
    try {
      // 克隆仓库
      await cloneRepo(repoPath, gitWorking, projectName, localPath);
      const gitRepo = git(path.join(localPath, projectName));
      // 选择某个tag，并checkoout
      const tagSelected = await requestTags(gitRepo, defaultTag);
      await checkoutTag(gitRepo, tagSelected);
      // 删除.git目录， 脱离
      fs.remove(path.join(localPath, projectName, '.git'));
      resolve(true);
    } catch (e) {
      reject(typeof e === 'string' ? e : 'Internal Error');
    }
  }));
